<?php
/**
 * DokuWiki Plugin mandates (Syntax Component)
 *
 * @license GPL 2 http://www.gnu.org/licenses/gpl-2.0.html
 * @author  kts <kts@exoss.net>
 */

// must be run within Dokuwiki
if (!defined('DOKU_INC')) die();

if (!defined('DOKU_LF')) define('DOKU_LF', "\n");
if (!defined('DOKU_TAB')) define('DOKU_TAB', "\t");
if (!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');

require_once DOKU_PLUGIN.'syntax.php';

class syntax_plugin_mandates_syntax extends DokuWiki_Syntax_Plugin {
    public function getType() {
        return 'substition';
    }

    public function getPType() {
        return 'normal';
    }

    public function getSort() {
        return 265;
    }


    public function connectTo($mode) {
        $this->Lexer->addSpecialPattern('mandate\{.+?\}',$mode,'plugin_mandates_syntax');
        $this->Lexer->addSpecialPattern('mandate_control\{[\w ]+\}',$mode,'plugin_mandates_syntax');
        $this->Lexer->addSpecialPattern('mandate_list\{[\w ]+\}',$mode,'plugin_mandates_syntax');
    }

    public function handle($match, $state, $pos, &$handler){
	$params = array();
	if ($match[7] == '_') {
		if ($match[8] == 'c') {
			$params["type"] = 2;
			$params["text"] = substr($match, 16, -1);
		} else if ($match[8] == 'l') {
			$params["type"] = 1;
			$params["text"] = substr($match, 13, -1);
		}
	} else {
		$params["type"] = 0;
		$params["text"] = substr($match, 8, -1);
	}
        return $params;
    }

	public function createControlForm($mandate_set) {
		$data = '<div class="mandates__control">'.
		'<span class="mandates__list_header">Current Mandates of \''.$mandate_set.'\'</span>'.
		'<form method="post" name="mandates__form" accept-charset=utf-8" class="mandates__form">'.
		'<table class="mandates__table">'.
		'<tr><th>name</th><th>description</th><th>delete</th></tr>';
		foreach($this->mandates as $name=>$description) {
			$id = $this->formatMandateId($name);
			$data .= '<tr><td><input name="mandates__id[]" class="mandates__input_id" type="input" value="'.$name.'"></input></td>'.
				'<td><textarea name="mandates__desc_'.$id.'" class="mandates__input_desc">'.$description.'</textarea></td>'.
				'<td><input type="checkbox" name="mandates__checkbox_'.$id.'"></input></td></tr>';
		}
		$data .= '<tr><td></td><td></td><td><input class="mandates__input_submit" type="submit" name="mandates__save" value="update"></input></td></tr>'.
			'</table>'.
			'<input type="hidden" name="mandates__set" value="'.$mandate_set.'"></input>'.
			'<span class="mandates__list_header">Add to \''.$mandate_set.'\'</span>';
		$data .= '<table class="mandates__table">'.
			'<tr><th>name</th><td><input name="mandates__input_id" class="mandates__input_id" type="input" value="some mandate"></input></td></tr>'.
			'<tr><th>description</th>'.
			'<td><textarea name="mandates__input_desc" class="mandates__input_desc">Description goes here.</textarea></td></tr>'.
			'<tr><td></td><td></td><td><input class="mandates__input_submit" type="submit" name="mandates__add" value="add"></input></td></tr>'.
			'</table>'.
			'<input type="hidden" name="mandates__set" value="'.$mandate_set.'"></input></form>'.
			'</div>';
		return $data;
	}

	public function addMandate() {
		$this->mandates[$_REQUEST['mandates__input_id']] = $_REQUEST['mandates__input_desc'];
		$this->setMandateSetData($_REQUEST['mandates__set']);
	}

	public function getMandate($name) {
		$id = $this->formatMandateId($name);
		if ($this->mandates[$id]) {
			return $this->mandates[$id];
		}
		return '';
	}

	public function getMandateSetData($mandate_set) {
		$data_file = $this->getMandateSetDataFile($mandate_set);
		$mandate_data = array();
		if (file_exists($data_file)) {
			$mandate_data = unserialize(file_get_contents($data_file));
		}
		return $mandate_data;
	}

	public function setMandateSetData($mandate_set) {
		if (!is_array($this->mandates)) return;
		$data_file = $this->getMandateSetDataFile($mandate_set);
		uksort($this->mandates, "strnatcasecmp");
		io_saveFile($data_file, serialize($this->mandates));
		return $data_file;
	}

	public function getMandateSetDataFile($mandate_set) {
		return metaFN( hsc( trim($mandate_set) ), '.mandates' );
	}

	public function formatMandateId($mandate_name) {
		return hsc( str_replace(' ', '_', $mandate_name) );
	}

	public $mandates = array();

    public function render($mode, &$renderer, $data) {
        if($mode != 'xhtml') return false;
	$value = '';
	$this->mandates = array();
	$renderer->info['cache'] = false;

	if ($data["type"] == 2) {
		// First handle submits
		if (!empty($_REQUEST['mandates__add'])) {
			$mandate_set = $_REQUEST['mandates__set'];
			$this->mandates = $this->getMandateSetData($mandate_set);
			$this->addMandate();
		} else if (!empty($_REQUEST['mandates__save'])) {
			$mandate_set = $_REQUEST['mandates__set'];
			$this->mandates = $this->getMandateSetData($mandate_set);
			foreach ($_REQUEST['mandates__id'] as $key=>$some_value) {
				$mandate_id = $this->formatMandateId($some_value);
				if (isset($_REQUEST['mandates__checkbox_'.$mandate_id])) {
					unset($this->mandates[$some_value]);
				} else {
					$this->mandates[$some_value] = $_REQUEST['mandates__desc_'.$mandate_id];
				}
			}
			$this->setMandateSetData($mandate_set);
		} else {
			$mandate_set = $data["text"];
			$this->mandates = $this->getMandateSetData($mandate_set);
		}
		$value .= $this->createControlForm($mandate_set);
	} else if ($data["type"] == 1) {
		$mandate_set = $data["text"];
		$this->mandates = $this->getMandateSetData($mandate_set);
		foreach($this->mandates as $mandate=>$desc) {
			$value .= '<ul><li><span class="mandates__entry_name">'.$mandate.'</span><ul><li class="mandates__entry_desc">'.nl2br($desc).'</li></ul></li></ul>';
		}
	} else if ($data["type"] == 0) {
		$current = 0;
		$text = $data["text"];
		while ($current < strlen($text)) {
			if ($text[$current] == ':') {
				$split = $current;
				break;
			}
			$current++;
		}
		if ($split) {
			$mandate_set = substr($text, 0, $split);
			$this->mandates = $this->getMandateSetData($mandate_set);
			$mandate_name = substr($text, $split+1);
			$value = '<span id="'.$mandate_name.'" class="mandates__entry_name">'.$mandate_name.'<span class="mandates__entry_text">'.nl2br($this->mandates[$mandate_name]).'</span></span>';
		} else {
			$value = 'no mandate set specified!';
		}
	}

	$renderer->doc .= $value;

        return true;
    }
}

// vim:ts=4:sw=4:et:
